/*  DO NOT EDIT THIS FILE.

    It has been auto-edited by fixincludes from:

	"fixinc/tests/inc/spawn.h"

    This had to be done to correct non-standard usages in the
    original, manufacturer supplied header file.  */



#if defined( SOLARIS_POSIX_SPAWN_RESTRICT_CHECK )
char *const *_RESTRICT_KYWD argv,
char *const *_RESTRICT_KYWD envp);
#endif  /* SOLARIS_POSIX_SPAWN_RESTRICT_CHECK */
