/*  DO NOT EDIT THIS FILE.

    It has been auto-edited by fixincludes from:

	"fixinc/tests/inc/rtldef/resolv.h"

    This had to be done to correct non-standard usages in the
    original, manufacturer supplied header file.  */



#if defined( VMS_DO_NOT_REDECLARE_HOSTALIAS_CHECK )
void		fp_nquery (const u_char *, int, FILE *);
/* __char_ptr32	hostalias (const char *); */

#endif  /* VMS_DO_NOT_REDECLARE_HOSTALIAS_CHECK */
