/*  DO NOT EDIT THIS FILE.

    It has been auto-edited by fixincludes from:

	"fixinc/tests/inc/stdarg.h"

    This had to be done to correct non-standard usages in the
    original, manufacturer supplied header file.  */



#if defined( OPENBSD_VA_START_CHECK )
#define va_start(v,l)   __builtin_va_start((v),l)
#endif  /* OPENBSD_VA_START_CHECK */
